;***************************** Librera "RS232.INC" *************************************
;
;	===================================================================
;	  Del libro "MICROCONTROLADOR PIC16F84. DESARROLLO DE PROYECTOS"
;	  E. Palacios, F. Remiro y L. Lpez.		www.pic16f84a.com
; 	  Editorial Ra-Ma.  www.ra-ma.es
;	===================================================================
;
; Estas subrutinas permiten realizar las tareas bsicas de control de la transmisin
; sere asincrona segn normas RS-232.
;
; Los parmetros adoptados para la comunicacin son los siguientes:
; - Velocidad de transmisin de 9600 baudios. La duracin de cada bit ser 104 s.
; - Un bit de inicio o Start a nivel bajo.
; - Dato de 8 bits.
; - Sin paridad.
; - Dos bits de final o Stop a nivel alto.
;
; El tiempo entre bit y bit  debe coincidir con el periodo de la seal leda o enviada.
; Como la velocidad de transmisin o recepcin es de 9600 baudios, el periodo ser:
; 1/9600 Baudios = 104 s. Se utilizar pues la subrutina Retardos_100micros.

	CBLOCK
	RS232_ContadorBits
	RS232_Dato
	ENDC

#DEFINE  RS232_Entrada	PORTA,4		; Lnea por la que se reciben los datos.
#DEFINE  RS232_Salida	PORTA,3		; Lnea por la que se envan los datos.
;
; Subrutina "RS232_Inicializa" -------------------------------------------------------------
;
; Configura las lneas de salida y entrada del microcontrolador.

RS232_Inicializa
	bsf	RS232_Salida				; Al principio salida en alto para resposo.
	bsf	STATUS,RP0
	bsf	RS232_Entrada				; Esta lnea se configura como entrada.
	bcf	RS232_Salida				; Esta lnea se configura como salida.
	bcf	STATUS,RP0
	return

; Subrutina "RS232_LeeDato" -------------------------------------------------------------
;
; El microcontrolador lee el dato por la lnea de entrada comenzando por el bit de menor
; peso. El dato ledo se enva finalmente en el registro de trabajo W.
;
; El ordenador parte siempre de un nivel alto, que es el estado que tiene cuando no 
; enva informacin. La secuencia utilizada es:
; 1	Espera que se ejecute el pulso negativo del bit Start o flanco de bajada.
; 2	Deja pasar un tiempo una y media veces mayor que el periodo de transmisin para
;		saltarse el bit de Start y lee el primer bit en su mitad.
; 3	Lee el resto de los bits de datos, esperando un tiempo igual a la duracin del
;		perodo entre lectura y lectura para testearlos en mitad del bit.
;
; Salida: En el registro de trabajo W el byte ledo.

RS232_LeeDato
	movlw	d'8'					; Nmero de bits a recibir.
	movwf	RS232_ContadorBits
RS232_EsperaBitStart
	btfsc	RS232_Entrada 			; Lee la entrada y espera a que sea "0".
	goto	RS232_EsperaBitStart	; No, pues espera el nivel bajo.
	call 	Retardo_100micros		; El primer bit debe leerlo un tiempo igual a una
	call 	Retardo_50micros 		; vez y media el periodo de transmisin.
RS232_LeeBit
	bcf		STATUS,C				; Ahora lee el pin. En principio supone que es 0.
	btfsc	RS232_Entrada			; Realmente es cero?
	bsf		STATUS,C				; No, pues cambia a "1".
	rrf		RS232_Dato,F			; Introduce el bit en el registro de lectura.
	call	Retardo_100micros		; Los siguientes bits los lee un periodo ms tarde.
	decfsz 	RS232_ContadorBits,F	; Comprueba que es el ltimo bit.
	goto	RS232_LeeBit			; Si no es el ltimo bit pasa a leer el siguiente.
	call	Retardo_200micros		; Espera un tiempo igual al los 2 bits de Stop.
	movf	RS232_Dato,W			; El resultado en el registro W.
	return	

; Subrutinas "RS232_EnviaDato" y "RS232_EnviaNmero" -------------------------------------
;
; El microcontrolador enva un dato por la lnea de salida comenzando por el bit de menor
; peso. En dato enviado ser el que le llegue a travs del registro de trabajo W.
; 1.	Enva un "0" durante un tiempo igual al periodo de la velocidad de transmisin.
;		Este es el bit de "Start".
; 2.	Enva el bit correspondiente:
; 		- Si va a enviar un "0" permanece en bajo durante el periodo correspondiente.
;		- Si va a escribir un "1" permanece en alto durante el periodo correspondiente.
; 3.	Enva dos bits "1" durante un tiempo igual al perodo de la velocidad de 
;		transmisin cada uno. Estos son los dos bits de Stop.
;
; Entrada:	En (W) el dato a enviar.

RS232_EnviaNumero					; Enva el cdigo ASCII de un nmero.
	addlw	'0'						; Lo pasa a cdigo ASCII sumndole el ASCII del 0.
RS232_EnviaDato
	movwf	RS232_Dato				; Guarda el contenido del byte a transmitir.
	movlw	d'8'					; Este es el nmero de bits a transmitir.
	movwf	RS232_ContadorBits
	bcf		RS232_Salida			; Bit de Start.
	call	Retardo_100micros
RS232_EnviaBit						; Comienza a enviar datos.
	rrf		RS232_Dato,F			; Lleva el bit que se quiere enviar al Carry para
	btfss	STATUS,C				; deducir su valor. Es un "1" el bit a transmitir?
	goto	RS232_EnviaCero			; No, pues enva un "0".
RS232_EnviaUno
	bsf		RS232_Salida			; Transmite un "1".
	goto	RS232_FinEnviaBit
RS232_EnviaCero
	bcf		RS232_Salida			; Transmite un "0".
RS232_FinEnviaBit
	call	Retardo_100micros		; Este es el tiempo que estar en alto o bajo.
	decfsz 	RS232_ContadorBits,F	; Comprueba que es el ltimo bit.
	goto	RS232_EnviaBit			; Como no es el ltimo bit repite la operacin.
	bsf		RS232_Salida			; Enva dos bits de Stop.
	call	Retardo_200micros
	return
	
;	===================================================================
;	  Del libro "MICROCONTROLADOR PIC16F84. DESARROLLO DE PROYECTOS"
;	  E. Palacios, F. Remiro y L. Lpez.		www.pic16f84a.com
; 	  Editorial Ra-Ma.  www.ra-ma.es
;	===================================================================
